/*
 * TopicListener.java
 *
 * Created on April 6, 2005, 10:52 AM
 */

package ch.epfl.lpd.ids.utils;

import javax.jms.*;

import java.util.Observable;
import java.util.Observer;

import ch.epfl.lpd.ids.serialization.IMessage;

/**
 * This class represents the implementation of a JMS message listener.
 * It is used for receiving asynchronously the JMS messages and then
 * dispatching them into the GUI of the client.
 */
public class TopicListener extends Observable implements MessageListener {
    
    /** 
     * Creates a new instance of TopicListener.
     * @param obs The observer object to add in order to notify it
     * when new messages are received.     
     */
    public TopicListener(Observer obs) {
        this.addObserver(obs);
    }    
    
    /**
     * Method called when receiving a new message from the JMS provider.
     * @param msg The received JMS message.
     */
    public void onMessage(Message msg) {
        try {
            // We only consider ObjectMessages
            if (msg instanceof ObjectMessage) {
                Object obj = ((ObjectMessage)msg).getObject();
                // We only consider ObjectMessage that contain IMessage.
                if (obj instanceof IMessage) {
                    // We notify the observers
                    this.setChanged();                                
                    this.notifyObservers((IMessage)obj);
                }
            }
        }
        catch (JMSException jmse) {
            IMessage imsg = new ch.epfl.lpd.ids.serialization.Message("A JMSException occured while getting a new JMS Message", jmse.getMessage().getBytes());
            this.setChanged();            
            this.notifyObservers(imsg);
            jmse.printStackTrace();
        }
    }
    
}
